#!/usr/bin/env python3
# coding=utf-8

import bisect
import json
from PyPDF2 import PdfReader
import os
import requests


url = 'https://winprotocoldoc.blob.core.windows.net/productionwindowsarchives/MS-LCID/[MS-LCID].pdf'
language_markers = [' Windows ', ' Release ']
bad_markers = ['release:', 'operating', 'server', 'first', 'supported', ' elk ', 'LCID support']
join_markers = ['Pseudo', 'Standard']
languages = set()
countries = set()


def is_language_line(phrase):
    return any(lm in phrase for lm in language_markers) and not any(bm in phrase.lower() for bm in bad_markers)


def cleanup(word):
    if word:
        word = word.replace(' -', '-')
        if word.startswith('('):
            word = word.replace('(','').replace(')','')
    return word


def extract_phrase_name_tag(phrase):
    data = phrase
    for lm in language_markers:
        data = data.split(lm)[0].strip()
    data = data.replace('Latin', '(Latin)').replace('((','(').replace('))',')').replace('),', ') ')
    es = [w.strip() for w in data.split('  ') if w.strip()]
    words = []
    for e in es:
        if ' 0x' in e:
            words += e.split()
        else:
            words += [e]
    if len(words) >= 3 and words[1].startswith('('): # ['Javanese', '(Latin)', ...
        words = [' '.join(words[:2])] + words[2:]
    if len(words) == 3: # "Zulu South Africa  0x0435  zu-ZA"
        if words[0].split()[0] in languages and not words[0].endswith(')'):
            l,_,c = words[0].partition(' ')
            words = [l,c] + words[1:]
        if 'City' in words[0] and len(words[0].split()) == 3: # "Latin Vatican City"
            words = words[0].split(maxsplit=1) + words[1:]
    if len(words) == 5: # "Spanish  United  States  0x540A  es-US"
        if words[0] in languages:
            words = words[:1] + [' '.join(words[1:3])] + words[3:]
        elif ' '.join(words[:2]) in languages:
            words = [' '.join(words[:2])] + words[2:]
    if len(words) == 1: # "MA Release 10"
        lang,country,tag = None, None, None
    elif len(words) == 3 and words[1].startswith('0x'):
        lang,country,tag = words[0], None, words[2]
    elif len(words) == 4 and words[2].startswith('0x'):
        lang = words[0]
        country = words[1].replace(' -', '-')
        tag = words[3]
    else:
        assert False, f'unknown phrase "{phrase}", {words}'
    lang,country = cleanup(lang),cleanup(country)
    if tag:
        tag = tag.replace(' ', '')
        if tag.endswith(','):
            tag = tag[:-1]
    # print(f'"{phrase}", {words}') # debug
    languages.add(lang)
    countries.add(country)
    name = f'{lang} - {country}' if country else lang
    return name,tag


def generate_page(phrases):
    name_tag = []
    for i,phrase in enumerate(phrases):
        # print(i, phrase) # debug
        if is_language_line(phrase):
            name,tag = extract_phrase_name_tag(phrase)
            if name and tag:
                name_tag.append((name, tag))
            print('.', flush=True, end='')
    return name_tag


def join_phrases(phrases):
    '''Join lines when possibly appropriate...'''
    complete_phrases = []
    prev = ''
    for phrase in phrases:
        phrase = phrase.strip()
        if phrase.startswith('(') or phrase.partition(' ')[0].endswith(')') \
                or (len(phrase.split()) <= 4 and is_language_line(phrase)) \
                or (is_language_line(phrase) and not is_language_line(prev) and any(jm in prev for jm in join_markers)):
            complete_phrases[-1] += ' ' + phrase
        else:
            complete_phrases.append(phrase)
        prev = phrase
    return complete_phrases


def write_output(name_tag):
    with open('bcp47.py', 'wt', encoding='utf-8') as w:
        print('# generated file', file=w)
        print('# coding=utf-8', file=w)
        print('languages = {', file=w)
        name_tag = [v for i,v in sorted(enumerate(name_tag), key=lambda ee: ee[0] - (1+len(ee[1][1])/20 if ee[1][0]==name_tag[ee[0]-1][0] and len(ee[1][1])<len(name_tag[ee[0]-1][1]) else 0.0))]
        namecache = set()
        for name,tag in name_tag:
            if name in namecache:
                name += ', ' + tag.split('-')[-1]
            assert name not in namecache, f'{name} already entered before'
            namecache.add(name)
            print('    "%s": "%s",' % (name, tag), file=w)
        print('}', file=w)
        print(file=w)
        print('tags = {v:k for k,v in languages.items()}', file=w)


# download and write local file
if not os.path.exists('.buffer.pdf'):
    print('Downloading the original masterpiece from Ms.Gates.')
    r = requests.get(url, stream=True)
    with open('.buffer.pdf', 'wb') as w:
        for chunk in r.iter_content(1024):
            w.write(chunk)

# open locally stored file and parse that
name_tag = [] # keeps name+tag tuples
with open('.buffer.pdf', 'rb') as r:
    reader = PdfReader(r)
    for page in reader.pages:
        phrases = page.extract_text().splitlines()
        phrases = join_phrases(phrases)
        name_tag += generate_page(phrases)
assert name_tag
write_output(name_tag)
print('file generation complete')



# -*- coding: utf-8 -*-
import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='bcp47',
    version='0.1.0',
    author='Jonas Byström',
    author_email='highfestiva@gmail.com',
    description='Language tags made easy',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/highfestiva/bcp47.py',
    packages=['bcp47'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



# generated file
# coding=utf-8
languages = {
    "Afar": "aa",
    "Afar - Djibouti": "aa-DJ",
    "Afar - Eritrea": "aa-ER",
    "Afar - Ethiopia": "aa-ET",
    "Afrikaans": "af",
    "Afrikaans - Namibia": "af-NA",
    "Afrikaans - South Africa": "af-ZA",
    "Aghem": "agq",
    "Aghem - Cameroon": "agq-CM",
    "Akan": "ak",
    "Akan - Ghana": "ak-GH",
    "Albanian": "sq",
    "Albanian - Albania": "sq-AL",
    "Albanian - North Macedonia": "sq-MK",
    "Alsatian": "gsw",
    "Alsatian - France": "gsw-FR",
    "Alsatian - Liechtenstein": "gsw-LI",
    "Alsatian - Switzerland": "gsw-CH",
    "Amharic": "am",
    "Amharic - Ethiopia": "am-ET",
    "Arabic": "ar",
    "Arabic - Algeria": "ar-DZ",
    "Arabic - Bahrain": "ar-BH",
    "Arabic - Chad": "ar-TD",
    "Arabic - Comoros": "ar-KM",
    "Arabic - Djibouti": "ar-DJ",
    "Arabic - Egypt": "ar-EG",
    "Arabic - Eritrea": "ar-ER",
    "Arabic - Iraq": "ar-IQ",
    "Arabic - Israel": "ar-IL",
    "Arabic - Jordan": "ar-JO",
    "Arabic - Kuwait": "ar-KW",
    "Arabic - Lebanon": "ar-LB",
    "Arabic - Libya": "ar-LY",
    "Arabic - Mauritania": "ar-MR",
    "Arabic - Morocco": "ar-MA",
    "Arabic - Oman": "ar-OM",
    "Arabic - Palestinian Authority": "ar-PS",
    "Arabic - Qatar": "ar-QA",
    "Arabic - Saudi Arabia": "ar-SA",
    "Arabic - Somalia": "ar-SO",
    "Arabic - South Sudan": "ar-SS",
    "Arabic - Sudan": "ar-SD",
    "Arabic - Syria": "ar-SY",
    "Arabic - Tunisia": "ar-TN",
    "Arabic - U.A.E.": "ar-AE",
    "Arabic - World": "ar-001",
    "Arabic - Yemen": "ar-YE",
    "Armenian": "hy",
    "Armenian - Armenia": "hy-AM",
    "Assamese": "as",
    "Assamese - India": "as-IN",
    "Asturian": "ast",
    "Asturian - Spain": "ast-ES",
    "Asu": "asa",
    "Asu - Tanzania": "asa-TZ",
    "Azerbaijani (Cyrillic)": "az-Cyrl",
    "Azerbaijani (Cyrillic) - Azerbaijan": "az-Cyrl-AZ",
    "Azerbaijani (Latin)": "az",
    "Azerbaijani (Latin), Latn": "az-Latn",
    "Azerbaijani (Latin) - Azerbaijan": "az-Latn-AZ",
    "Bafia": "ksf",
    "Bafia - Cameroon": "ksf-CM",
    "Bamanankan": "bm",
    "Bamanankan (Latin) - Mali": "bm-Latn-ML",
    "Bangla": "bn",
    "Bangla - Bangladesh": "bn-BD",
    "Bangla - India": "bn-IN",
    "Basaa": "bas",
    "Basaa - Cameroon": "bas-CM",
    "Bashkir": "ba",
    "Bashkir - Russia": "ba-RU",
    "Basque": "eu",
    "Basque - Spain": "eu-ES",
    "Belarusian": "be",
    "Belarusian - Belarus": "be-BY",
    "Bemba": "bem",
    "Bemba - Zambia": "bem-ZM",
    "Bena": "bez",
    "Bena - Tanzania": "bez-TZ",
    "Bhojpuri": "bho",
    "Bhojpuri (Devanagari)": "bho-Deva",
    "Bhojpuri (Devanagari) - India": "bho-Deva-IN",
    "Blin": "byn",
    "Blin - Eritrea": "byn-ER",
    "Bodo": "brx",
    "Bodo - India": "brx-IN",
    "Bosnian (Cyrillic)": "bs-Cyrl",
    "Bosnian (Cyrillic) - Bosnia and Herzegovina": "bs-Cyrl-BA",
    "Bosnian (Latin)": "bs",
    "Bosnian (Latin), Latn": "bs-Latn",
    "Bosnian (Latin) - Bosnia and Herzegovina": "bs-Latn-BA",
    "Breton": "br",
    "Breton - France": "br-FR",
    "Bulgarian": "bg",
    "Bulgarian - Bulgaria": "bg-BG",
    "Burmese": "my",
    "Burmese - Myanmar": "my-MM",
    "Catalan": "ca",
    "Catalan - Andorra": "ca-AD",
    "Catalan - France": "ca-FR",
    "Catalan - Italy": "ca-IT",
    "Catalan - Spain": "ca-ES",
    "Cebuano": "ceb",
    "Cebuan (Latin)": "ceb-Latn",
    "Cebuan (Latin) - Philippines": "ceb-Latn-PH",
    "Central Atlas Tamazight (Arabic) - Morocco": "tzm-Arab-MA",
    "Central Atlas Tamazight (Latin) - Morocco": "tzm-Latn-MA",
    "Central Kurdish": "ku",
    "Central Kurdish, Arab": "ku-Arab",
    "Central Kurdish - Iraq": "ku-Arab-IQ",
    "Chakma": "ccp",
    "Chakma - Chakma": "ccp-Cakm",
    "Chakma - Bangladesh": "ccp-Cakm-BD",
    "Chakma - India": "ccp-Cakm-IN",
    "Chechen - Russia": "ce-RU",
    "Cherokee": "chr",
    "Cherokee, Cher": "chr-Cher",
    "Cherokee - United States": "chr-Cher-US",
    "Chiga": "cgg",
    "Chiga - Uganda": "cgg-UG",
    "Chinese (Simplified)": "zh",
    "Chinese (Simplified), Hans": "zh-Hans",
    "Chinese (Simplified) - People's Republic of China": "zh-CN",
    "Chinese (Simplified) - Singapore": "zh-SG",
    "Chinese (Traditional)": "zh-Hant",
    "Chinese (Traditional) - Hong Kong S.A.R.": "zh-HK",
    "Chinese (Traditional) - Macao S.A.R.": "zh-MO",
    "Chinese (Traditional) - Taiwan": "zh-TW",
    "Church Slavic - Russia": "cu-RU",
    "Chuvash": "cv",
    "Chuvash (Cyrillic)": "cv-Cyrl",
    "Chuvash (Cyrillic) - Russia": "cv-Cyrl-RU",
    "Congo Swahili": "swc",
    "Congo Swahili - Congo DRC": "swc-CD",
    "Cornish": "kw",
    "Cornish - United Kingdom": "kw-GB",
    "Corsican": "co",
    "Corsican - France": "co-FR",
    "Croatian": "hr",
    "Croatian - Croatia": "hr-HR",
    "Croatian (Latin) - Bosnia and Herzegovina": "hr-BA",
    "Czech": "cs",
    "Czech - Czech Republic": "cs-CZ",
    "Danish": "da",
    "Danish - Denmark": "da-DK",
    "Danish - Greenland": "da-GL",
    "Dari": "prs",
    "Dari - Afghanistan": "prs-AF",
    "Divehi": "dv",
    "Divehi - Maldives": "dv-MV",
    "Duala": "dua",
    "Duala - Cameroon": "dua-CM",
    "Dutch": "nl",
    "Dutch - Aruba": "nl-AW",
    "Dutch - Belgium": "nl-BE",
    "Dutch - Bonaire, Sint Eustatius and Saba": "nl-BQ",
    "Dutch - Curaçao": "nl-CW",
    "Dutch - Netherlands": "nl-NL",
    "Dutch - Sint Maarten": "nl-SX",
    "Dutch - Suriname": "nl-SR",
    "Dzongkha": "dz",
    "Dzongkha - Bhutan": "dz-BT",
    "Embu": "ebu",
    "Embu - Kenya": "ebu-KE",
    "English": "en",
    "English - American Samoa": "en-AS",
    "English - Anguilla": "en-AI",
    "English - Antigua and Barbuda": "en-AG",
    "English - Australia": "en-AU",
    "English - Austria": "en-AT",
    "English - Bahamas": "en-BS",
    "English - Barbados": "en-BB",
    "English - Belgium": "en-BE",
    "English - Belize": "en-BZ",
    "English - Bermuda": "en-BM",
    "English - Botswana": "en-BW",
    "English - British Indian Ocean Territory": "en-IO",
    "English - British Virgin Islands": "en-VG",
    "English - Burundi": "en-BI",
    "English - Cameroon": "en-CM",
    "English - Canada": "en-CA",
    "English - Caribbean": "en-029",
    "English - Cayman Islands": "en-KY",
    "English - Christmas Island": "en-CX",
    "English - Cocos [Keeling] Islands": "en-CC",
    "English - Cook Islands": "en-CK",
    "English - Cyprus": "en-CY",
    "English - Denmark": "en-DK",
    "English - Dominica": "en-DM",
    "English - Eritrea": "en-ER",
    "English - Europe": "en-150",
    "English - Falkland Islands": "en-FK",
    "English - Finland": "en-FI",
    "English - Fiji": "en-FJ",
    "English - Gambia": "en-GM",
    "English - Germany": "en-DE",
    "English - Ghana": "en-GH",
    "English - Gibraltar": "en-GI",
    "English - Grenada": "en-GD",
    "English - Guam": "en-GU",
    "English - Guernsey": "en-GG",
    "English - Guyana": "en-GY",
    "English - Hong Kong": "en-HK",
    "English - India": "en-IN",
    "English - Ireland": "en-IE",
    "English - Isle of Man": "en-IM",
    "English - Israel": "en-IL",
    "English - Jamaica": "en-JM",
    "English - Jersey": "en-JE",
    "English - Kenya": "en-KE",
    "English - Kiribati": "en-KI",
    "English - Lesotho": "en-LS",
    "English - Liberia": "en-LR",
    "English - Macao SAR": "en-MO",
    "English - Madagascar": "en-MG",
    "English - Malawi": "en-MW",
    "English - Malaysia": "en-MY",
    "English - Maldives": "en-MV",
    "English - Malta": "en-MT",
    "English - Marshall Islands": "en-MH",
    "English - Mauritius": "en-MU",
    "English - Micronesia": "en-FM",
    "English - Montserrat": "en-MS",
    "English - Namibia": "en-NA",
    "English - Nauru": "en-NR",
    "English - Netherlands": "en-NL",
    "English - New Zealand": "en-NZ",
    "English - Nigeria": "en-NG",
    "English - Niue": "en-NU",
    "English - Norfolk Island": "en-NF",
    "English - Northern Mariana Islands": "en-MP",
    "English - Pakistan": "en-PK",
    "English - Palau": "en-PW",
    "English - Papua New Guinea": "en-PG",
    "English - Pitcairn Islands": "en-PN",
    "English - Puerto Rico": "en-PR",
    "English - Republic of the Philippines": "en-PH",
    "English - Rwanda": "en-RW",
    "English - Saint Kitts and Nevis": "en-KN",
    "English - Saint Lucia": "en-LC",
    "English - Saint Vincent and the Grenadines": "en-VC",
    "English - Samoa": "en-WS",
    "English - Seychelles": "en-SC",
    "English - Sierra Leone": "en-SL",
    "English - Singapore": "en-SG",
    "English - Sint Maarten": "en-SX",
    "English - Slovenia": "en-SI",
    "English - Solomon Islands": "en-SB",
    "English - South Africa": "en-ZA",
    "English - South Sudan": "en-SS",
    "Cunha": "en-SH",
    "English - Sudan": "en-SD",
    "English - Swaziland": "en-SZ",
    "English - Sweden": "en-SE",
    "English - Switzerland": "en-CH",
    "English - Tanzania": "en-TZ",
    "English - Tokelau": "en-TK",
    "English - Tonga": "en-TO",
    "English - Trinidad and Tobago": "en-TT",
    "English - Turks and Caicos Islands": "en-TC",
    "English - Tuvalu": "en-TV",
    "English - Uganda": "en-UG",
    "English - United Arab Emirates": "en-AE",
    "English - United Kingdom": "en-GB",
    "English - United States": "en-US",
    "English - US Minor Outlying Islands": "en-UM",
    "English - US Virgin Islands": "en-VI",
    "English - Vanuatu": "en-VU",
    "English - World": "en-001",
    "English - Zambia": "en-ZM",
    "English - Zimbabwe": "en-ZW",
    "Esperanto": "eo",
    "Esperanto - World": "eo-001",
    "Estonian": "et",
    "Estonian - Estonia": "et-EE",
    "Ewe": "ee",
    "Ewe - Ghana": "ee-GH",
    "Ewe - Togo": "ee-TG",
    "Ewondo": "ewo",
    "Ewondo - Cameroon": "ewo-CM",
    "Faroese": "fo",
    "Faroese - Denmark": "fo-DK",
    "Faroese - Faroe Islands": "fo-FO",
    "Filipino": "fil",
    "Filipino - Philippines": "fil-PH",
    "Finnish": "fi",
    "Finnish - Finland": "fi-FI",
    "French": "fr",
    "French - Algeria": "fr-DZ",
    "French - Belgium": "fr-BE",
    "French - Benin": "fr-BJ",
    "French - Burkina Faso": "fr-BF",
    "French - Burundi": "fr-BI",
    "French - Cameroon": "fr-CM",
    "French - Canada": "fr-CA",
    "French - Caribbean": "fr-029",
    "French - Central African Republic": "fr-CF",
    "French - Chad": "fr-TD",
    "French - Comoros": "fr-KM",
    "French - Congo": "fr-CG",
    "French - Congo, DRC": "fr-CD",
    "French - Côte d'Ivoire": "fr-CI",
    "French - Djibouti": "fr-DJ",
    "French - Equatorial Guinea": "fr-GQ",
    "French - France": "fr-FR",
    "French - French Guiana": "fr-GF",
    "French - French Polynesia": "fr-PF",
    "French - Gabon": "fr-GA",
    "French - Guadeloupe": "fr-GP",
    "French - Guinea": "fr-GN",
    "French - Haiti": "fr-HT",
    "French - Luxembourg": "fr-LU",
    "French - Madagascar": "fr-MG",
    "French - Mali": "fr-ML",
    "French - Martinique": "fr-MQ",
    "French - Mauritania": "fr-MR",
    "French - Mauritius": "fr-MU",
    "French - Mayotte": "fr-YT",
    "French - Morocco": "fr-MA",
    "French - New Caledonia": "fr-NC",
    "French - Niger": "fr-NE",
    "French - Principality of Monaco": "fr-MC",
    "French - Reunion": "fr-RE",
    "French - Rwanda": "fr-RW",
    "French - Saint Barthélemy": "fr-BL",
    "French - Saint Martin": "fr-MF",
    "French - Saint Pierre and Miquelon": "fr-PM",
    "French - Senegal": "fr-SN",
    "French - Seychelles": "fr-SC",
    "French - Switzerland": "fr-CH",
    "French - Syria": "fr-SY",
    "French - Togo": "fr-TG",
    "French - Tunisia": "fr-TN",
    "French - Vanuatu": "fr-VU",
    "French - Wallis and Futuna": "fr-WF",
    "Frisian": "fy",
    "Frisian - Netherlands": "fy-NL",
    "Friulian": "fur",
    "Friulian - Italy": "fur-IT",
    "Fulah": "ff",
    "Fulah (Latin)": "ff-Latn",
    "Fulah (Latin) - Burkina Faso": "ff-Latn-BF",
    "Fulah - Cameroon": "ff-CM",
    "Fulah (Latin) - Cameroon": "ff-Latn-CM",
    "Fulah (Latin) - Gambia": "ff-Latn-GM",
    "Fulah (Latin) - Ghana": "ff-Latn-GH",
    "Fulah - Guinea": "ff-GN",
    "Fulah (Latin) - Guinea": "ff-Latn-GN",
    "Fulah (Latin) - Guinea-Bissau": "ff-Latn-GW",
    "Fulah (Latin) - Liberia": "ff-Latn-LR",
    "Fulah - Mauritania": "ff-MR",
    "Fulah (Latin) - Mauritania": "ff-Latn-MR",
    "Fulah (Latin) - Niger": "ff-Latn-NE",
    "Fulah - Nigeria": "ff-NG",
    "Fulah (Latin) - Nigeria": "ff-Latn-NG",
    "Fulah - Senegal": "ff-Latn-SN",
    "Fulah (Latin) - Sierra Leone": "ff-Latn-SL",
    "Galician": "gl",
    "Galician - Spain": "gl-ES",
    "Ganda": "lg",
    "Ganda - Uganda": "lg-UG",
    "Georgian": "ka",
    "Georgian - Georgia": "ka-GE",
    "German": "de",
    "German - Austria": "de-AT",
    "German - Belgium": "de-BE",
    "German - Germany": "de-DE",
    "German - Italy": "de-IT",
    "German - Liechtenstein": "de-LI",
    "German - Luxembourg": "de-LU",
    "German - Switzerland": "de-CH",
    "Greek": "el",
    "Greek - Cyprus": "el-CY",
    "Greek - Greece": "el-GR",
    "Greenlandic": "kl",
    "Greenlandic - Greenland": "kl-GL",
    "Guarani": "gn",
    "Guarani - Paraguay": "gn-PY",
    "Gujarati": "gu",
    "Gujarati - India": "gu-IN",
    "Gusii": "guz",
    "Gusii - Kenya": "guz-KE",
    "Haryanvi": "bgc",
    "Haryanvi (Devanagari)": "bgc-Deva",
    "Haryanvi (Devanagari) - India": "bgc-Deva-IN",
    "Hausa (Latin)": "ha",
    "Hausa (Latin), Latn": "ha-Latn",
    "Hausa (Latin) - Ghana": "ha-Latn-GH",
    "Hausa (Latin) - Niger": "ha-Latn-NE",
    "Hausa (Latin) - Nigeria": "ha-Latn-NG",
    "Hawaiian": "haw",
    "Hawaiian - United States": "haw-US",
    "Hebrew": "he",
    "Hebrew - Israel": "he-IL",
    "Hindi": "hi",
    "Hindi - India": "hi-IN",
    "Hindi (Latin)": "hi-Latn",
    "Hindi (Latin) - India": "hi-Latn-IN",
    "Hungarian": "hu",
    "Hungarian - Hungary": "hu-HU",
    "Icelandic": "is",
    "Icelandic - Iceland": "is-IS",
    "Igbo": "ig",
    "Igbo - Nigeria": "ig-NG",
    "Indonesian": "id",
    "Indonesian - Indonesia": "id-ID",
    "Interlingua": "ia",
    "Interlingua - France": "ia-FR",
    "Interlingua - World": "ia-001",
    "Inuktitut (Latin)": "iu",
    "Inuktitut (Latin), Latn": "iu-Latn",
    "Inuktitut (Latin) - Canada": "iu-Latn-CA",
    "Inuktitut (Syllabics)": "iu-Cans",
    "Inuktitut (Syllabics) - Canada": "iu-Cans-CA",
    "Irish": "ga",
    "Irish - Ireland": "ga-IE",
    "Italian": "it",
    "Italian - Italy": "it-IT",
    "Italian - San Marino": "it-SM",
    "Italian - Switzerland": "it-CH",
    "Italian - Vatican City": "it-VA",
    "Japanese": "ja",
    "Japanese - Japan": "ja-JP",
    "Javanese": "jv",
    "Javanese (Latin)": "jv-Latn",
    "Javanese (Latin) - Indonesia": "jv-Latn-ID",
    "Jola-Fonyi": "dyo",
    "Jola-Fonyi - Senegal": "dyo-SN",
    "Kabuverdianu": "kea",
    "Kabuverdianu - Cabo Verde": "kea-CV",
    "Kabyle": "kab",
    "Kabyle - Algeria": "kab-DZ",
    "Kaingang": "kgp",
    "Kaingang (Latin)": "kgp-Latn",
    "Kaingang (Latin) - Brazil": "kgp-Latn-BR",
    "Kako": "kkj",
    "Kako - Cameroon": "kkj-CM",
    "Kalenjin": "kln",
    "Kalenjin - Kenya": "kln-KE",
    "Kamba": "kam",
    "Kamba - Kenya": "kam-KE",
    "Kannada": "kn",
    "Kannada - India": "kn-IN",
    "Kanuri (Latin) - Nigeria": "kr-Latn-NG",
    "Kashmiri": "ks",
    "Kashmiri - Perso-Arabic": "ks-Arab",
    "Kashmiri - Perso-Arabic, IN": "ks-Arab-IN",
    "Kashmiri (Devanagari) - India": "ks-Deva-IN",
    "Kazakh": "kk",
    "Kazakh - Kazakhstan": "kk-KZ",
    "Khmer": "km",
    "Khmer - Cambodia": "km-KH",
    "K'iche": "quc",
    "K'iche - Guatemala": "quc-Latn-GT",
    "Kikuyu": "ki",
    "Kikuyu - Kenya": "ki-KE",
    "Kinyarwanda": "rw",
    "Kinyarwanda - Rwanda": "rw-RW",
    "Kiswahili": "sw",
    "Kiswahili - Kenya": "sw-KE",
    "Kiswahili - Tanzania": "sw-TZ",
    "Kiswahili - Uganda": "sw-UG",
    "Konkani": "kok",
    "Konkani - India": "kok-IN",
    "Korean": "ko",
    "Korean - Korea": "ko-KR",
    "Korean - North Korea": "ko-KP",
    "Koyra Chiini": "khq",
    "Koyra Chiini - Mali": "khq-ML",
    "Koyraboro Senni": "ses",
    "Koyraboro Senni - Mali": "ses-ML",
    "Kwasio": "nmg",
    "Kwasio - Cameroon": "nmg-CM",
    "Kyrgyz": "ky",
    "Kyrgyz - Kyrgyzstan": "ky-KG",
    "Kurdish - Perso-Arabic, Iran": "ku-Arab-IR",
    "Lakota": "lkt",
    "Lakota - United States": "lkt-US",
    "Langi": "lag",
    "Langi - Tanzania": "lag-TZ",
    "Lao": "lo",
    "Lao - Lao P.D.R.": "lo-LA",
    "Latin - Vatican City": "la-VA",
    "Latvian": "lv",
    "Latvian - Latvia": "lv-LV",
    "Lingala": "ln",
    "Lingala - Angola": "ln-AO",
    "Lingala - Central African Republic": "ln-CF",
    "Lingala - Congo": "ln-CG",
    "Lingala - Congo DRC": "ln-CD",
    "Lithuanian": "lt",
    "Lithuanian - Lithuania": "lt-LT",
    "Low German": "nds",
    "Low German - Germany": "nds-DE",
    "Low German - Netherlands": "nds-NL",
    "Lower Sorbian": "dsb",
    "Lower Sorbian - Germany": "dsb-DE",
    "Luba-Katanga": "lu",
    "Luba-Katanga - Congo DRC": "lu-CD",
    "Luo": "luo",
    "Luo - Kenya": "luo-KE",
    "Luxembourgish": "lb",
    "Luxembourgish - Luxembourg": "lb-LU",
    "Luyia": "luy",
    "Luyia - Kenya": "luy-KE",
    "Macedonian": "mk",
    "Macedonian - North Macedonia": "mk-MK",
    "Machame": "jmc",
    "Machame - Tanzania": "jmc-TZ",
    "Makhuwa-Meetto": "mgh",
    "Makhuwa-Meetto - Mozambique": "mgh-MZ",
    "Makonde": "kde",
    "Makonde - Tanzania": "kde-TZ",
    "Malagasy": "mg",
    "Malagasy - Madagascar": "mg-MG",
    "Malay": "ms",
    "Malay - Brunei Darussalam": "ms-BN",
    "Malay - Malaysia": "ms-MY",
    "Malayalam": "ml",
    "Malayalam - India": "ml-IN",
    "Maltese": "mt",
    "Maltese - Malta": "mt-MT",
    "Manx": "gv",
    "Manx - Isle of Man": "gv-IM",
    "Maori": "mi",
    "Maori - New Zealand": "mi-NZ",
    "Mapudungun": "arn",
    "Mapudungun - Chile": "arn-CL",
    "Marathi": "mr",
    "Marathi - India": "mr-IN",
    "Masai": "mas",
    "Masai - Kenya": "mas-KE",
    "Masai - Tanzania": "mas-TZ",
    "Mazanderani - Iran": "mzn-IR",
    "Meru": "mer",
    "Meru - Kenya": "mer-KE",
    "Meta'": "mgo",
    "Meta' - Cameroon": "mgo-CM",
    "Mohawk": "moh",
    "Mohawk - Canada": "moh-CA",
    "Mongolian (Cyrillic)": "mn",
    "Mongolian (Cyrillic), Cyrl": "mn-Cyrl",
    "Mongolian (Cyrillic) - Mongolia": "mn-MN",
    "Mongolian (Traditional Mongolian)": "mn-Mong",
    "Mongolian (Traditional Mongolian) - People's Republic of China": "mn-Mong-CN",
    "Mongolian (Traditional Mongolian) - Mongolia": "mn-Mong-MN",
    "Morisyen": "mfe",
    "Morisyen - Mauritius": "mfe-MU",
    "Mundang": "mua",
    "Mundang - Cameroon": "mua-CM",
    "N'ko": "nqo",
    "N'ko - Guinea": "nqo-GN",
    "Nama": "naq",
    "Nama - Namibia": "naq-NA",
    "Nepali": "ne",
    "Nepali - India": "ne-IN",
    "Nepali - Nepal": "ne-NP",
    "Ngiemboon": "nnh",
    "Ngiemboon - Cameroon": "nnh-CM",
    "Ngomba": "jgo",
    "Ngomba - Cameroon": "jgo-CM",
    "Nheengatu": "yrl",
    "Nheengatu (Latin)": "yrl-Latn",
    "Nheengatu (Latin) - Brazil": "yrl-Latn-BR",
    "Nheengatu (Latin) - Colombia": "yrl-Latn-CO",
    "Nheengatu (Latin) - Venezuela": "yrl-Latn-VE",
    "Northern Luri - Iraq": "lrc-IQ",
    "Northern Luri - Iran": "lrc-IR",
    "North Ndebele": "nd",
    "North Ndebele - Zimbabwe": "nd-ZW",
    "Norwegian (Bokmal)": "no",
    "Norwegian (Bokmal), nb": "nb",
    "Norwegian (Bokmal) - Norway": "nb-NO",
    "Norwegian (Nynorsk)": "nn",
    "Norwegian (Nynorsk) - Norway": "nn-NO",
    "Norwegian Bokmål - Svalbard and Jan Mayen": "nb-SJ",
    "Nuer": "nus",
    "Nuer - Sudan": "nus-SD",
    "Nuer - South Sudan": "nus-SS",
    "Nyankole": "nyn",
    "Nyankole - Uganda": "nyn-UG",
    "Occitan": "oc",
    "Occitan - France": "oc-FR",
    "Occitan - Spain": "oc-ES",
    "Odia": "or",
    "Odia - India": "or-IN",
    "Oromo": "om",
    "Oromo - Ethiopia": "om-ET",
    "Oromo - Kenya": "om-KE",
    "Ossetian": "os",
    "Ossetian - Cyrillic, Georgia": "os-GE",
    "Ossetian - Cyrillic, Russia": "os-RU",
    "Pashto": "ps",
    "Pashto - Afghanistan": "ps-AF",
    "Pashto - Pakistan": "ps-PK",
    "Persian": "fa",
    "Persian - Afghanistan": "fa-AF",
    "Persian - Iran": "fa-IR",
    "Polish": "pl",
    "Polish - Poland": "pl-PL",
    "Portuguese": "pt",
    "Portuguese - Angola": "pt-AO",
    "Portuguese - Brazil": "pt-BR",
    "Portuguese - Cabo Verde": "pt-CV",
    "Portuguese - Equatorial Guinea": "pt-GQ",
    "Portuguese - Guinea-Bissau": "pt-GW",
    "Portuguese - Luxembourg": "pt-LU",
    "Portuguese - Macao SAR": "pt-MO",
    "Portuguese - Mozambique": "pt-MZ",
    "Portuguese - Portugal": "pt-PT",
    "Portuguese - São Tomé and Príncipe": "pt-ST",
    "Portuguese - Switzerland": "pt-CH",
    "Portuguese - Timor-Leste": "pt-TL",
    "Prussian": "prg-001",
    "Pseudo Language - Pseudo locale for east Asian/complex script localization testing": "qps-ploca",
    "Pseudo Language - Pseudo locale used for localization testing": "qps-ploc",
    "Pseudo Language - Pseudo locale used for localization testing of mirrored locales": "qps-plocm",
    "Punjabi": "pa",
    "Punjabi, Arab": "pa-Arab",
    "Punjabi - India": "pa-IN",
    "Punjabi - Islamic Republic of Pakistan": "pa-Arab-PK",
    "Quechua": "quz",
    "Quechua - Bolivia": "quz-BO",
    "Quechua - Ecuador": "quz-EC",
    "Quechua - Peru": "quz-PE",
    "Rajasthani": "raj",
    "Rajasthani (Devanagari)": "raj-Deva",
    "Rajasthani (Devanagari) - India": "raj-Deva-IN",
    "Ripuarian": "ksh",
    "Ripuarian - Germany": "ksh-DE",
    "Romanian": "ro",
    "Romanian - Moldova": "ro-MD",
    "Romanian - Romania": "ro-RO",
    "Romansh": "rm",
    "Romansh - Switzerland": "rm-CH",
    "Rombo": "rof",
    "Rombo - Tanzania": "rof-TZ",
    "Rundi": "rn",
    "Rundi - Burundi": "rn-BI",
    "Russian": "ru",
    "Russian - Belarus": "ru-BY",
    "Russian - Kazakhstan": "ru-KZ",
    "Russian - Kyrgyzstan": "ru-KG",
    "Russian - Moldova": "ru-MD",
    "Russian - Russia": "ru-RU",
    "Russian - Ukraine": "ru-UA",
    "Rwa": "rwk",
    "Rwa - Tanzania": "rwk-TZ",
    "Saho": "ssy",
    "Saho - Eritrea": "ssy-ER",
    "Sakha": "sah",
    "Sakha - Russia": "sah-RU",
    "Samburu": "saq",
    "Samburu - Kenya": "saq-KE",
    "Sami (Inari)": "smn",
    "Sami (Inari) - Finland": "smn-FI",
    "Sami (Lule)": "smj",
    "Sami (Lule) - Norway": "smj-NO",
    "Sami (Lule) - Sweden": "smj-SE",
    "Sami (Northern)": "se",
    "Sami (Northern) - Finland": "se-FI",
    "Sami (Northern) - Norway": "se-NO",
    "Sami (Northern) - Sweden": "se-SE",
    "Sami (Skolt)": "sms",
    "Sami (Skolt) - Finland": "sms-FI",
    "Sami (Southern)": "sma",
    "Sami (Southern) - Norway": "sma-NO",
    "Sami (Southern) - Sweden": "sma-SE",
    "Sango": "sg",
    "Sango - Central African Republic": "sg-CF",
    "Sangu": "sbp",
    "Sangu - Tanzania": "sbp-TZ",
    "Sanskrit": "sa",
    "Sanskrit - India": "sa-IN",
    "Sardinian": "sc",
    "Sardinian (Latin)": "sc-Latn",
    "Sardinian (Latin) - Italy": "sc-Latn-IT",
    "Scottish Gaelic": "gd",
    "Scottish Gaelic - United Kingdom": "gd-GB",
    "Sena": "seh",
    "Sena - Mozambique": "seh-MZ",
    "Serbian (Cyrillic)": "sr-Cyrl",
    "Serbian (Cyrillic) - Bosnia and Herzegovina": "sr-Cyrl-BA",
    "Serbian (Cyrillic) - Montenegro": "sr-Cyrl-ME",
    "Serbian (Cyrillic) - Serbia": "sr-Cyrl-RS",
    "Serbian (Cyrillic) - Serbia and Montenegro (Former)": "sr-Cyrl-CS",
    "Serbian (Latin)": "sr",
    "Serbian (Latin), Latn": "sr-Latn",
    "Serbian (Latin) - Bosnia and Herzegovina": "sr-Latn-BA",
    "Serbian (Latin) - Montenegro": "sr-Latn-ME",
    "Serbian (Latin) - Serbia": "sr-Latn-RS",
    "Serbian (Latin) - Serbia and Montenegro (Former)": "sr-Latn-CS",
    "Sesotho sa Leboa": "nso",
    "Sesotho sa Leboa - South Africa": "nso-ZA",
    "Setswana": "tn",
    "Setswana - Botswana": "tn-BW",
    "Setswana - South Africa": "tn-ZA",
    "Shambala": "ksb",
    "Shambala - Tanzania": "ksb-TZ",
    "Shona": "sn",
    "Shona (Latin)": "sn-Latn",
    "Shona - Zimbabwe": "sn-Latn-ZW",
    "Sindhi": "sd",
    "Sindhi, Arab": "sd-Arab",
    "Sindhi - Islamic Republic of Pakistan": "sd-Arab-PK",
    "Sinhala": "si",
    "Sinhala - Sri Lanka": "si-LK",
    "Slovak": "sk",
    "Slovak - Slovakia": "sk-SK",
    "Slovenian": "sl",
    "Slovenian - Slovenia": "sl-SI",
    "Soga": "xog",
    "Soga - Uganda": "xog-UG",
    "Somali": "so",
    "Somali - Djibouti": "so-DJ",
    "Somali - Ethiopia": "so-ET",
    "Somali - Kenya": "so-KE",
    "Somali - Somalia": "so-SO",
    "Sotho": "st",
    "Sotho - South Africa": "st-ZA",
    "South Ndebele": "nr",
    "South Ndebele - South Africa": "nr-ZA",
    "Southern Sotho - Lesotho": "st-LS",
    "Spanish": "es",
    "Spanish - Argentina": "es-AR",
    "Spanish - Belize": "es-BZ",
    "Spanish - Bolivarian Republic of Venezuela": "es-VE",
    "Spanish - Bolivia": "es-BO",
    "Spanish - Brazil": "es-BR",
    "Spanish - Chile": "es-CL",
    "Spanish - Colombia": "es-CO",
    "Spanish - Costa Rica": "es-CR",
    "Spanish - Cuba": "es-CU",
    "Spanish - Dominican Republic": "es-DO",
    "Spanish - Ecuador": "es-EC",
    "Spanish - El Salvador": "es-SV",
    "Spanish - Equatorial Guinea": "es-GQ",
    "Spanish - Guatemala": "es-GT",
    "Spanish - Honduras": "es-HN",
    "Spanish - Latin America": "es-419",
    "Spanish - Mexico": "es-MX",
    "Spanish - Nicaragua": "es-NI",
    "Spanish - Panama": "es-PA",
    "Spanish - Paraguay": "es-PY",
    "Spanish - Peru": "es-PE",
    "Spanish - Philippines": "es-PH",
    "Spanish - Puerto Rico": "es-PR",
    "Spanish - Spain": "es-ES",
    "Spanish - Spain, ES_tradnl": "es-ES_tradnl",
    "Spanish - United States": "es-US",
    "Spanish - Uruguay": "es-UY",
    "Standard Moroccan Tamazight": "zgh",
    "Standard Moroccan Tamazight - Morocco": "zgh-Tfng-MA",
    "Standard Moroccan Tamazight - Tifinagh": "zgh-Tfng",
    "Swati": "ss",
    "Swati - South Africa": "ss-ZA",
    "Swati - Swaziland": "ss-SZ",
    "Swedish": "sv",
    "Swedish - Åland Islands": "sv-AX",
    "Swedish - Finland": "sv-FI",
    "Swedish - Sweden": "sv-SE",
    "Syriac": "syr",
    "Syriac - Syria": "syr-SY",
    "Tachelhit": "shi",
    "Tachelhit - Tifinagh": "shi-Tfng",
    "Tachelhit - Tifinagh, Morocco": "shi-Tfng-MA",
    "Tachelhit (Latin)": "shi-Latn",
    "Tachelhit (Latin) - Morocco": "shi-Latn-MA",
    "Taita": "dav",
    "Taita - Kenya": "dav-KE",
    "Tajik (Cyrillic)": "tg",
    "Tajik (Cyrillic), Cyrl": "tg-Cyrl",
    "Tajik (Cyrillic) - Tajikistan": "tg-Cyrl-TJ",
    "Tamazight (Latin)": "tzm",
    "Tamazight (Latin), Latn": "tzm-Latn",
    "Tamazight (Latin) - Algeria": "tzm-Latn-DZ",
    "Tamil": "ta",
    "Tamil - India": "ta-IN",
    "Tamil - Malaysia": "ta-MY",
    "Tamil - Singapore": "ta-SG",
    "Tamil - Sri Lanka": "ta-LK",
    "Tasawaq": "twq",
    "Tasawaq - Niger": "twq-NE",
    "Tatar": "tt",
    "Tatar - Russia": "tt-RU",
    "Telugu": "te",
    "Telugu - India": "te-IN",
    "Teso": "teo",
    "Teso - Kenya": "teo-KE",
    "Teso - Uganda": "teo-UG",
    "Thai": "th",
    "Thai - Thailand": "th-TH",
    "Tibetan": "bo",
    "Tibetan - India": "bo-IN",
    "Tibetan - People's Republic of China": "bo-CN",
    "Tigre": "tig",
    "Tigre - Eritrea": "tig-ER",
    "Tigrinya": "ti",
    "Tigrinya - Eritrea": "ti-ER",
    "Tigrinya - Ethiopia": "ti-ET",
    "Tongan": "to",
    "Tongan - Tonga": "to-TO",
    "Tsonga": "ts",
    "Tsonga - South Africa": "ts-ZA",
    "Turkish": "tr",
    "Turkish - Cyprus": "tr-CY",
    "Turkish - Turkey": "tr-TR",
    "Turkmen": "tk",
    "Turkmen - Turkmenistan": "tk-TM",
    "Ukrainian": "uk",
    "Ukrainian - Ukraine": "uk-UA",
    "Upper Sorbian": "hsb",
    "Upper Sorbian - Germany": "hsb-DE",
    "Urdu": "ur",
    "Urdu - India": "ur-IN",
    "Urdu - Islamic Republic of Pakistan": "ur-PK",
    "Uyghur": "ug",
    "Uyghur - People's Republic of China": "ug-CN",
    "Uzbek - Perso-Arabic": "uz-Arab",
    "Uzbek - Perso-Arabic, Afghanistan": "uz-Arab-AF",
    "Uzbek (Cyrillic)": "uz-Cyrl",
    "Uzbek (Cyrillic) - Uzbekistan": "uz-Cyrl-UZ",
    "Uzbek (Latin)": "uz",
    "Uzbek (Latin), Latn": "uz-Latn",
    "Uzbek (Latin) - Uzbekistan": "uz-Latn-UZ",
    "Vai": "vai",
    "Vai, Vaii": "vai-Vaii",
    "Vai - Liberia": "vai-Vaii-LR",
    "Vai (Latin) - Liberia": "vai-Latn-LR",
    "Vai (Latin)": "vai-Latn",
    "Valencian - Spain": "ca-ES-valencia",
    "Venda": "ve",
    "Venda - South Africa": "ve-ZA",
    "Vietnamese": "vi",
    "Vietnamese - Vietnam": "vi-VN",
    "Volapük": "vo",
    "Volapük - World": "vo-001",
    "Vunjo": "vun",
    "Vunjo - Tanzania": "vun-TZ",
    "Walser": "wae",
    "Walser - Switzerland": "wae-CH",
    "Welsh": "cy",
    "Welsh - United Kingdom": "cy-GB",
    "Wolaytta": "wal",
    "Wolaytta - Ethiopia": "wal-ET",
    "Wolof": "wo",
    "Wolof - Senegal": "wo-SN",
    "Xhosa": "xh",
    "Xhosa - South Africa": "xh-ZA",
    "Yangben": "yav",
    "Yangben - Cameroon": "yav-CM",
    "Yi": "ii",
    "Yi - People's Republic of China": "ii-CN",
    "Yiddish - World": "yi-001",
    "Yoruba": "yo",
    "Yoruba - Benin": "yo-BJ",
    "Yoruba - Nigeria": "yo-NG",
    "Zarma": "dje",
    "Zarma - Niger": "dje-NE",
    "Zulu": "zu",
    "Zulu - South Africa": "zu-ZA",
}

tags = {v:k for k,v in languages.items()}
